@@ -31,7 +31,7 @@ import string
 supports_alpha = False
 
 HOME = os.getenv('HOME')
-APP_VERSION = "2.1"
+APP_VERSION = "2.2"
 
 if not os.path.exists(HOME + "/.touchsearch/"):
    dir = '%s/.touchsearch/' % HOME
@@ -45,22 +45,43 @@ class SearchEngines:
          self.search_engines = cPickle.load(data)
          data.close()
       except Exception:
-         self.search_engines = [
-            {"name":"Google","url":"http://www.google.com/custom?hl=en&client=pub-2014082016242195&channel=0312482696&cof=FORID%%3A13%%3BAH%%3Aleft%%3BS%%3Ahttp%%3A%%2F%%2Fwww.google.com%%2Fcse%%3Fcx%%3D003942025946635967052%%253A4eq_cwvmcim%%26ie%%3DUTF-8%%26q%%3D%%26sa%%3DSearch%%3BCX%%3AWeb%%2520%%2528mobile-optimized%%2529%%3BL%%3Ahttp%%3A%%2F%%2Fwww.google.com%%2Fimages%%2Fgoogle_sm.gif%%3BLH%%3A59%%3BLP%%3A1%%3BVLC%%3A%%23551a8b%%3BGFNT%%3A%%23666666%%3BDIV%%3A%%23cccccc%%3B&adkw=AELymgXvPYDM4X5-BVw_BuVC9v6xuBytBCsp1DhSSByJlGcS0stpcK38sTNjYyfwouQDi7IW3HBDDErXgreOqmYQtxp2DU96bhRZ5Nyjxv8CHhWCKzlYWUc&btnG=Search&cx=partner-pub-2014082016242195&q=%s","icon":"google.png"},
-            {"name":"Wikipedia","url":"http://en.wikipedia.org/w/index.php?title=Special%%3ASearch&search=%s&fulltext=Search","icon":"wikipedia.png"},
-            {"name":"eBay","url":"http://shop.ebay.com/?_from=R40&_trksid=p3907.m38.l1313&_nkw=%s&=&_sacat=See-All-Categories","icon":"ebay.png"},
-            {"name":"Google Maps","url":"http://tomch.com/maemaps.html?from=%s&go=1","icon":"google_maps.png"},
-            {"name":"Amazon","url":"http://www.amazon.com/s/ref=nb_ss?url=search-alias%%3Daps&field-keywords=%s&x=0&y=0","icon":"amazon.png"},
-            {"name":"Dictionary.com","url":"http://m.reference.com/d/search.html?q=%s","icon":"dictionary.png"},
-            {"name":"IMDb","url":"http://www.imdb.com/find?s=all&q=%s","icon":"imdb.png"},
-            {"name":"YouTube","url":"http://www.youtube.com/results?search_query=%s&search_type=&aq=f","icon":"youtube.png"},
-            {"name":"WebMD","url":"http://www.webmd.com/search/search_results/default.aspx?query=%s&sourceType=undefined","icon":"webmd.png"}
-            ]
+         self.search_engines = self.load_default()
          self.write(self.search_engines)
 
    def read(self):
       return self.search_engines
       
+   def load_default(self):
+      search_engines = [
+            {"name":"Google","url":"http://www.google.com/custom?hl=en&client=pub-2014082016242195&channel=0643327877&cof=FORID%%3A13%%3BAH%%3Aleft%%3BS%%3Ahttp%%3A%%2F%%2Fwww.google.com%%2Fcse%%3Fcx%%3D003942025946635967052%%253A4eq_cwvmcim%%26ie%%3DUTF-8%%26q%%3D%%26sa%%3DSearch%%3BCX%%3AWeb%%2520%%2528mobile-optimized%%2529%%3BL%%3Ahttp%%3A%%2F%%2Fwww.google.com%%2Fimages%%2Fgoogle_sm.gif%%3BLH%%3A59%%3BLP%%3A1%%3BVLC%%3A%%23551a8b%%3BGFNT%%3A%%23666666%%3BDIV%%3A%%23cccccc%%3B&adkw=AELymgXvPYDM4X5-BVw_BuVC9v6xuBytBCsp1DhSSByJlGcS0stpcK38sTNjYyfwouQDi7IW3HBDDErXgreOqmYQtxp2DU96bhRZ5Nyjxv8CHhWCKzlYWUc&btnG=Search&cx=partner-pub-2014082016242195&q=%s","icon":"google.png","prefix":None},
+            {"name":"QuickBar","url":"None","icon":"quickbar.png","prefix":None},
+            {"name":"Wikipedia","url":"http://en.wikipedia.org/w/index.php?title=Special%%3ASearch&search=%s&fulltext=Search","icon":"wikipedia.png","prefix":"w"},
+            {"name":"eBay","url":"http://shop.ebay.com/?_from=R40&_trksid=p3907.m38.l1313&_nkw=%s&=&_sacat=See-All-Categories","icon":"ebay.png","prefix":"e"},
+            {"name":"Google Maps Mobile","url":"http://www.google.com/maps/m#ll=&q=%s","icon":"google_maps.png","prefix":"gm"},
+            {"name":"Amazon","url":"http://www.amazon.com/s/ref=nb_ss?url=search-alias%%3Daps&field-keywords=%s&x=0&y=0","icon":"amazon.png","prefix":"a"},
+            {"name":"Dictionary.com","url":"http://m.reference.com/d/search.html?q=%s","icon":"dictionary.png","prefix":"d"},
+            {"name":"IMDb","url":"http://www.imdb.com/find?s=all&q=%s","icon":"imdb.png","prefix":"i"},
+            {"name":"YouTube","url":"http://www.youtube.com/results?search_query=%s&search_type=&aq=f","icon":"youtube.png","prefix":"y"},
+            {"name":"WebMD","url":"http://www.webmd.com/search/search_results/default.aspx?query=%s&sourceType=undefined","icon":"webmd.png","prefix":"wm"}
+            ]
+      return search_engines
+      
+   def merge(self):
+      default = self.load_default()
+      current = self.read()
+
+      for d_engine in default:
+         not_in = True
+         for c_engine in current:
+            if d_engine['name'] == c_engine['name']:
+               not_in = False
+         if not_in:
+            current.append(d_engine)
+      self.write(current)
+      
+   def reset(self):
+      self.write(self.load_default())
+      
    def write(self, new_search_engines):
       data = open(HOME + '/.touchsearch/search_engines.data', 'w')
       cPickle.dump(new_search_engines, data)
@@ -188,10 +209,10 @@ class TouchSearchHomePlugin(hildondesktop.HomePluginItem):
       print self.history
       self.image, self.url = self.get_data(self.settings['search_engine'])
       
-      self.s_eventbox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
-      self.s_eventbox.connect("button-press-event", self.search_dialog, self.osso_rpc)
-      self.e_eventbox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
-      self.e_eventbox.connect("button-press-event", self.select_engine_dialog)
+      self.s_eventbox.set_events(gtk.gdk.BUTTON_RELEASE_MASK)
+      self.s_eventbox.connect("button-release-event", self.search_dialog, self.osso_rpc)
+      self.e_eventbox.set_events(gtk.gdk.BUTTON_RELEASE_MASK)
+      self.e_eventbox.connect("button-release-event", self.select_engine_dialog)
 
       self.hbox.pack_start(self.s_eventbox, False, False, self.settings['padding'])
       self.hbox.pack_end(self.e_eventbox, False, False, self.settings['padding'])
@@ -299,7 +320,7 @@ class TouchSearchHomePlugin(hildondesktop.HomePluginItem):
       settings_button.connect('clicked', self.show_settings)
       
       configure_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
-      configure_button.set_text("Configure", "Add, Edit, Delete and Sort search engines")
+      configure_button.set_text("Configure", "Customize your search experience")
       configure_button.set_alignment(0,0,0,0)
       configure_button.connect('clicked', self.show_configure)
       
@@ -406,6 +427,48 @@ License along with this program.  If not, see
       dialog.destroy()
       
    def show_configure(self, widget):
+      dialog = gtk.Dialog("Configure Search Engines", None, gtk.DIALOG_DESTROY_WITH_PARENT)
+      
+      customize_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+      customize_button.set_text("Customize", "Add, Edit, Delete and Sort search engines")
+      customize_button.set_alignment(0,0,0,0)
+      customize_button.connect('clicked', self.show_customize)
+      
+      merge_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+      merge_button.set_text("Merge", "Add default search engines to current database")
+      merge_button.set_alignment(0,0,0,0)
+      merge_button.connect('clicked', self.show_merge)
+      
+      reset_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+      reset_button.set_text("Reset", "Reset search engine database to default state")
+      reset_button.set_alignment(0,0,0,0)
+      reset_button.connect('clicked', self.show_reset)
+      
+      dialog.vbox.pack_start(customize_button, True, True, 0)
+      dialog.vbox.pack_start(merge_button, True, True, 0)
+      dialog.vbox.pack_start(reset_button, True, True, 0)
+      
+      dialog.show_all()
+      dialog.run()
+      dialog.destroy()
+      
+   def show_merge(self, widget):
+      note = hildon.Note("confirmation", gtk.Window(), "Merging adds any default search engines not in your current database to your search engine list. This is useful if you accidentally deleted a default search engine, or a search engine has been added in a future revision of TouchSearch that is not currently present in your database. Continue?")
+      retcode = gtk.Dialog.run(note)
+      if retcode == gtk.RESPONSE_OK:
+         SearchEngines().merge()
+      note.destroy()
+      
+   def show_reset(self, widget):
+      note = hildon.Note("confirmation", gtk.Window(), """This will reset your search engine database to it's default "factory" state which will undo all customizations. Continue?""")
+      retcode = gtk.Dialog.run(note)
+      if retcode == gtk.RESPONSE_OK:
+         SearchEngines().reset()
+      note.destroy()
+      
+   def show_customize(self, widget):
+      # default to Google
+      Settings().write(widget, "Google")
       dialog = gtk.Dialog("Configure Search Engines", None, gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR)
       #dialog.set_size_request(-1, 380)
       #done_button = dialog.add_button("Done", gtk.RESPONSE_OK)
@@ -461,6 +524,8 @@ License along with this program.  If not, see
       dialog.run()
       dialog.destroy()
       
+      self.update_info()
+      
    def get_treeview_data(self):
       selection = self.treeview.get_selection()
       model, treeiter = selection.get_selected()
@@ -495,9 +560,12 @@ License along with this program.  If not, see
       else:
          down_button.set_sensitive(True)
       
-      if model[treeiter][0] == "Google" or model[treeiter][0] == self.settings['search_engine']: # Un-editable default search engine
+      if model[treeiter][0] == "Google":
          edit_button.set_sensitive(False)
          delete_button.set_sensitive(False)
+      elif model[treeiter][0] == "QuickBar":
+         edit_button.set_sensitive(False)
+         delete_button.set_sensitive(True)
       else:
          edit_button.set_sensitive(True)
          delete_button.set_sensitive(True)
@@ -550,22 +618,32 @@ License along with this program.  If not, see
       name_entry.set_text(engine['name'])
       url_entry = hildon.Entry(gtk.HILDON_SIZE_AUTO)
       url_entry.set_text(engine['url'])
+      prefix_entry = hildon.Entry(gtk.HILDON_SIZE_AUTO)
+      prefix_entry.set_text(engine['prefix'])
+      prefix_entry.set_max_length(3)
+      
+      name_hbox = gtk.HBox()
+      name_hbox.pack_start(gtk.Label("Name:"), False, False, 5)
+      name_hbox.pack_end(name_entry, True, True, 5)
+      
+      url_hbox = gtk.HBox()
+      url_hbox.pack_start(gtk.Label("URL:"), False, False, 5)
+      url_hbox.pack_end(url_entry, True, True, 5)
+      
+      prefix_hbox = gtk.HBox()
+      prefix_hbox.pack_start(gtk.Label("QuickBar Prefix:"), False, False, 5)
+      prefix_hbox.pack_end(prefix_entry, True, True, 5)
       
       file_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
       file_button.set_text("Icon:", engine['icon'])
       file_button.connect('clicked', self.file_button_callback, engine['icon'])
       
-      dialog.vbox.pack_start(gtk.Label("Name:"), False, False, 5)
-      dialog.vbox.pack_start(name_entry, True, True, 5)
-      dialog.vbox.pack_start(gtk.HSeparator())
-      
-      dialog.vbox.pack_start(gtk.Label("URL:"), False, False, 5)
-      dialog.vbox.pack_start(url_entry, True, True, 5)
-      dialog.vbox.pack_start(gtk.HSeparator())
-      
+      dialog.vbox.pack_start(name_hbox, True, True, 5)
+      dialog.vbox.pack_start(url_hbox, True, True, 5)
+      dialog.vbox.pack_start(prefix_hbox, True, True, 5)
       dialog.vbox.pack_start(file_button, True, True, 5)
       
-      ok.connect('clicked', self.ok_button_callback, name_entry, url_entry, file_button, engine)
+      ok.connect('clicked', self.ok_button_callback, name_entry, url_entry, prefix_entry, file_button, engine)
       dialog.show_all()
       dialog.run()
       dialog.destroy()
@@ -610,7 +688,11 @@ License along with this program.  If not, see
       url_entry2 = hildon.Entry(gtk.HILDON_SIZE_AUTO)
       url2_hbox.pack_start(gtk.Label("Search String:"), False, False, 5)
       url2_hbox.pack_end(url_entry2, True, True, 5)
-      
+      prefix_hbox = gtk.HBox()
+      prefix_entry = hildon.Entry(gtk.HILDON_SIZE_AUTO)
+      prefix_entry.set_max_length(3)
+      prefix_hbox.pack_start(gtk.Label("QuickBar Prefix:"), False, False, 5)
+      prefix_hbox.pack_end(prefix_entry, True, True, 5)
       
       file_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
       file_button.set_title("Icon:")
@@ -618,6 +700,7 @@ License along with this program.  If not, see
       
       third_vbox.pack_start(name_hbox, True, True, 10)
       third_vbox.pack_start(url2_hbox, True, True, 10)
+      third_vbox.pack_start(prefix_hbox, True, True, 10)
       third_vbox.pack_start(file_button, True, True, 10)
       
       finish_label = gtk.Label("All required information has been inputted. Tap Finish to add your newly-created search engine to TouchSearch!")
@@ -631,7 +714,7 @@ License along with this program.  If not, see
       
       wizard_dialog = hildon.WizardDialog(None, "Add New Search Engine", notebook)
       wizard_dialog.set_forward_page_func(self.check_go_forward, (url_entry, query_entry, name_entry, url_entry2, file_button))
-      notebook.connect("switch-page", self.on_page_switch, url_entry, query_entry, name_entry, url_entry2, file_button)
+      notebook.connect("switch-page", self.on_page_switch, url_entry, query_entry, name_entry, url_entry2, prefix_entry, file_button)
       wizard_dialog.show_all()
       response = wizard_dialog.run()
       wizard_dialog.destroy()
@@ -689,12 +772,12 @@ License along with this program.  If not, see
       dialog.destroy()
       
    def actually_add_it(self):
-      engine = {'name':self.w_name,'url':self.w_url2,'icon':self.w_file}
+      engine = {'name':self.w_name,'url':self.w_url2,'icon':self.w_file,'prefix':self.w_prefix}
       self.search_engines.append(engine)
       SearchEngines().write(self.search_engines)
       self.liststore.append([engine['name']])
       
-   def on_page_switch(self, widget, page, num, url_entry, query_entry, name_entry, url_entry2, file_button):
+   def on_page_switch(self, widget, page, num, url_entry, query_entry, name_entry, url_entry2, prefix_entry, file_button):
       """This function is responsible for getting user data from the wizard dialog"""
       
       if num == 2:
@@ -707,6 +790,7 @@ License along with this program.  If not, see
       elif num == 3:
          self.w_name = name_entry.get_text()
          self.w_url2 = url_entry2.get_text()
+         self.w_prefix = prefix_entry.get_text()
          self.w_file = file_button.get_value()
          
    def check_go_forward(self, widget, current, data):
@@ -725,15 +809,17 @@ License along with this program.  If not, see
       else:
          return True
       
-   def ok_button_callback(self, widget, name_entry, url_entry, file_button, engine):
+   def ok_button_callback(self, widget, name_entry, url_entry, prefix_entry, file_button, engine):
       model, treeiter, engine = self.get_treeview_data()
       name = name_entry.get_text()
       url = url_entry.get_text()
+      prefix = prefix_entry.get_text()
       icon = file_button.get_value()
       
       self.liststore[treeiter][0] = name
       self.search_engines[engine]['name'] = name
       self.search_engines[engine]['url'] = url
+      self.search_engines[engine]['prefix'] = prefix
       self.search_engines[engine]['icon'] = icon
       
       SearchEngines().write(self.search_engines)
@@ -760,29 +846,39 @@ License along with this program.  If not, see
       dialog = gtk.Dialog("Search for", None, gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR)
       search_button = dialog.add_button("Search", 1)
 
-      image, url = self.get_data(self.settings['search_engine'])
+      self.image, self.url = self.get_data(self.settings['search_engine'])
+      evbox = gtk.EventBox()
+      evbox.add(self.image)
+      evbox.set_events(gtk.gdk.BUTTON_RELEASE_MASK)
+      evbox.connect("button-release-event", self.select_engine_dialog, dialog)
       entry = hildon.Entry(0)
-      completion = gtk.EntryCompletion()
-      self.liststore = gtk.ListStore(str)
-      for i in self.history:
-         self.liststore.append([i])
-      completion.set_model(self.liststore)
-      entry.set_completion(completion)
-      completion.set_text_column(0)
+      #completion = gtk.EntryCompletion()
+      #self.liststore = gtk.ListStore(str)
+      #for i in self.history:
+      #   self.liststore.append([i])
+      #completion.set_model(self.liststore)
+      #entry.set_completion(completion)
+      #completion.set_text_column(0)
 
       hbox = gtk.HBox()
-      hbox.pack_start(image, False, False, 5)
+      hbox.pack_start(evbox, False, False, 5)
       hbox.pack_end(entry, True, True, 5)
       
-      search_button.connect('clicked', self.search_the_web, entry, url, osso_rpc)
-      entry.connect("key-press-event", self.key_pressed, dialog, entry, url, osso_rpc)
+      search_button.connect('clicked', self.search_the_web, entry, self.url, osso_rpc)
+      entry.connect("key-press-event", self.key_pressed, dialog, entry, self.url, osso_rpc)
       dialog.vbox.pack_start(hbox)
       dialog.show_all()
       dialog.run()
 
       dialog.destroy()
       
-   def select_engine_dialog(self, widget, data=None):
+   def update_search_icon(self, widget, data=None):
+      widget.remove(widget.get_child())
+      self.image, self.url = self.get_data(self.settings['search_engine'])
+      widget.add(self.image)
+      self.image.show_all()
+      
+   def select_engine_dialog(self, widget, data=None, search_dialog=None):
       dialog = hildon.PickerDialog(gtk.Window())
       dialog.set_title("Select Search Engine")
       
@@ -797,16 +893,38 @@ License along with this program.  If not, see
       response = dialog.run()
       dialog.destroy()
       
+      print search_dialog
+      if isinstance(search_dialog, gtk.Dialog):
+         search_dialog.destroy()
+         self.search_dialog(None, None, self.osso_rpc)
+      
    def search_the_web(self, widget, entry, url, osso_rpc):
       query = entry.get_text()
-      url_scheme = url % query
+      if url == "None": # QuickBar
+         prefix = query.split()[0]
+         for s in self.search_engines:
+            try:
+               if s['prefix'] == prefix:
+                  engine = s
+            except KeyError:
+               pass # old search engine files won't have prefixes
+            if s['name'] == "Google":
+               google_engine = s['url']
+         query_without_prefix = query[len(prefix)+1:]
+         try:
+            url_scheme = engine['url'] % query_without_prefix
+         except UnboundLocalError:
+            url_scheme = google_engine % query
+      else:
+         url_scheme = url % query
       print url_scheme
+      print type(url_scheme)
       osso_rpc.rpc_run_with_defaults("osso_browser", "open_new_window", (url_scheme,))
       # History isn't working at the moment
       #History().update(query)
       
    def key_pressed(self, widget, event, dialog, entry, url, osso_rpc, *args):
-      if event.keyval == gtk.keysyms.KP_Enter:
+      if event.keyval == gtk.keysyms.KP_Enter or event.keyval == gtk.keysyms.Return:
          self.search_the_web(widget, entry, url, osso_rpc)
          dialog.destroy()
       else:
